<?php
// $Id: install.oracle.inc,v 1.1.2.11 2011/01/15 14:15:05 andreagariboldi Exp $

define('DRUPAL_ROOT', '/var/www/html/drupal');
include_once DRUPAL_ROOT.'/includes/database.oracle.inc';

// Oracle specific install functions
openlog("database.oracle.log", LOG_PID | LOG_PERROR, LOG_LOCAL0);
/**
 * Check if Oracle is available.
 *
 * @return
 *  TRUE/FALSE
 */
function oracle_is_available() {
  foreach (PDO::getAvailableDrivers() as $driver)
     if ($driver=='oci')
       return true; 
	
  return false;
}

/**
 * Check if we can connect to Oracle.
 *
 * @return
 *  TRUE/FALSE
 */
function drupal_test_oracle($url, &$success)
{
	  global $active_db;
	  
	  if (!oracle_is_available())
	  {
	    drupal_set_message(st('PHP Oracle support not enabled.'), 'error');
	    return false;
	  }
	
	  $url = parse_url($url);
	  $conn_string = '';
	
	  $url['user'] = urldecode($url['user']);
	  $url['pass'] = isset($url['pass']) ? urldecode($url['pass']) : '';
	  $url['host'] = urldecode($url['host']);
	  $url['path'] = urldecode($url['path']);
	
	  if (isset($url['port'])) 
	    $url['host'] = $url['host'] .':'. $url['port'];
	  
	
	  $url['host'] = "//" . $url['host'] . $url['path']; 
	
	  $dsn = 'oci:dbname=' . $url['host'] . ';charset=AL32UTF8';
	  syslog(LOG_ERR,"dsn: ".$dsn);
	  $conn= false;
	  $success = array();
	  
	  try
	  {
		  $conn= new PDO($dsn, $url['user'], $url['pass']);
	
		  $conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		  $conn->setAttribute(PDO::ATTR_CASE, PDO::CASE_LOWER);
		  $conn->setAttribute(PDO::ATTR_STRINGIFY_FETCHES, TRUE);
		  
		  $active_db= $conn;
          $success[] = 'CONNECT';
	  }
	  catch (Exception $ex)
	  {
          drupal_set_message(st('Failed to connect to your Oracle database server. Oracle reports the following message: %error.<ul><li>Are you sure you have the correct username and password?</li><li>Are you sure that you have typed the correct database hostname?</li><li>Are you sure that the database server is running?</li><li>Are you sure you typed the correct database name?</li></ul>For more help, see the <a href="http://drupal.org/node/258">Installation and upgrading handbook</a>. If you are unsure what these terms mean you should probably contact your hosting provider.', array('%error' => $ex->getMessage())), 'error');
          return false;
	  }
	  
	  try
	  {
		  // Test CREATE.
		  
	  	  $query= 'CREATE TABLE drupal_install_test (id number NOT NULL)';
	  	  
		  oracle_query($query);
		  
		  $success[] = 'CREATE';
	  }
	  catch (Exception $ex)
	  {
          drupal_set_message(st('Failed to create a test table on your Oracle database server with the command %query. Oracle reports the following message: %error.<ul><li>Are you sure the configured username has the necessary Oracle permissions to create tables in the database?</li></ul>For more help, see the <a href="http://drupal.org/node/258">Installation and upgrading handbook</a>. If you are unsure what these terms mean you should probably contact your hosting provider.', array('%query' => $query, '%error' => $ex->getMessage())), 'error');
          return false;
	  }
	  
	  $active_db->beginTransaction();
	  
	  try
	  {
		  // Test INSERT.
		  
	  	  $query= 'INSERT INTO drupal_install_test (id) VALUES (1)';
	  	  
		  oracle_query($query);
		  
		  $success[] = 'INSERT';
	  }
	  catch (Exception $ex)
	  {
          drupal_set_message(st('Failed to insert a value into a test table on your Oracle database server. We tried inserting a value with the command %query and Oracle reported the following error: %error.', array('%query' => $query, '%error' => $ex->getMessage())), 'error');
          return false;
	  }
	  
	  try
	  {
		  // Test SELECT.
		  
	  	  $query= 'select * from drupal_install_test';
	  	  
		  $obj= oracle_query($query)->fetchObject();
		  
		  $success[] = 'SELECT';
	  }
	  catch (Exception $ex)
	  {
          drupal_set_message(st('Failed to select a test table on your Oracle database server with the command %query. Oracle reports the following message: %error.<ul><li>Are you sure the configured username has the necessary Oracle permissions to create tables in the database?</li></ul>For more help, see the <a href="http://drupal.org/node/258">Installation and upgrading handbook</a>. If you are unsure what these terms mean you should probably contact your hosting provider.', array('%query' => $query, '%error' => $ex->getMessage())), 'error');
          return false;
	  }
	  

	  try
	  {
		  // Test UPDATE.
		  
	  	  $query= 'UPDATE drupal_install_test SET id = 2';
	  	  
		  $obj= oracle_query($query);
		  
		  $success[] = 'UPDATE';
	  }
	  catch (Exception $ex)
	  {
          drupal_set_message(st('Failed to update a test table on your Oracle database server with the command %query. Oracle reports the following message: %error.<ul><li>Are you sure the configured username has the necessary Oracle permissions to create tables in the database?</li></ul>For more help, see the <a href="http://drupal.org/node/258">Installation and upgrading handbook</a>. If you are unsure what these terms mean you should probably contact your hosting provider.', array('%query' => $query, '%error' => $ex->getMessage())), 'error');
          return false;
	  }
	  
	  
	  try
	  {
		  // Test LOCK.
		  
	  	  $query= 'LOCK TABLE drupal_install_test IN SHARE ROW EXCLUSIVE MODE';
	  	  
		  $obj= oracle_query($query);
		  
		  $success[] = 'UPDATE';
	  }
	  catch (Exception $ex)
	  {
          drupal_set_message(st('Failed to lock a test table on your Oracle database server with the command %query. Oracle reports the following message: %error.<ul><li>Are you sure the configured username has the necessary Oracle permissions to create tables in the database?</li></ul>For more help, see the <a href="http://drupal.org/node/258">Installation and upgrading handbook</a>. If you are unsure what these terms mean you should probably contact your hosting provider.', array('%query' => $query, '%error' => $ex->getMessage())), 'error');
          return false;
	  }
	  
	  try
	  {
		  // Test UNLOCK.
		  
	  	  $query= 'COMMIT';
	  	  
    	  $conn->commit(); 
          $success[] = 'UNLOCK';
          
	  }
	  catch (Exception $ex)
	  {
          drupal_set_message(st('Failed to lock a test table on your Oracle database server with the command %query. Oracle reports the following message: %error.<ul><li>Are you sure the configured username has the necessary Oracle permissions to create tables in the database?</li></ul>For more help, see the <a href="http://drupal.org/node/258">Installation and upgrading handbook</a>. If you are unsure what these terms mean you should probably contact your hosting provider.', array('%query' => $query, '%error' => $ex->getMessage())), 'error');
          return false;
	  }
  

	  try
	  {
		  // Test UPDATE.
		  
	  	  $query= 'DELETE FROM drupal_install_test';
	  	  
		  $obj= oracle_query($query);
		  
		  $success[] = 'DELETE';
	  }
	  catch (Exception $ex)
	  {
          drupal_set_message(st('Failed to delete a test table on your Oracle database server with the command %query. Oracle reports the following message: %error.<ul><li>Are you sure the configured username has the necessary Oracle permissions to create tables in the database?</li></ul>For more help, see the <a href="http://drupal.org/node/258">Installation and upgrading handbook</a>. If you are unsure what these terms mean you should probably contact your hosting provider.', array('%query' => $query, '%error' => $ex->getMessage())), 'error');
          return false;
	  }
	  

	  try
	  {
		  // Test DROP.
		  
	  	  $query= 'DROP TABLE drupal_install_test';
	  	  
		  $obj= oracle_query($query);
		  
		  $success[] = 'DROP';
	  }
	  catch (Exception $ex)
	  {
          drupal_set_message(st('Failed to drop a test table on your Oracle database server with the command %query. Oracle reports the following message: %error.<ul><li>Are you sure the configured username has the necessary Oracle permissions to create tables in the database?</li></ul>For more help, see the <a href="http://drupal.org/node/258">Installation and upgrading handbook</a>. If you are unsure what these terms mean you should probably contact your hosting provider.', array('%query' => $query, '%error' => $ex->getMessage())), 'error');
          return false;
	  }
	  
	  // create driver needed objects
	  
	  oracle_create_failsafe_objects("includes/oracle/table");
	  oracle_create_failsafe_objects("includes/oracle/index");
	  oracle_create_failsafe_objects("includes/oracle/sequence");
	  oracle_create_objects("includes/oracle/function");
	  oracle_create_objects("includes/oracle/procedure");
	  oracle_create_sp_objects("includes/oracle/type");
	  oracle_create_sp_objects("includes/oracle/package");
      oracle_query("begin dbms_utility.compile_schema(user); end;");
      
      return true;
}

function oracle_create_sp_objects($dir_path)
{
    $dir = opendir($dir_path);
    
    while($name = readdir($dir))
    {
         if (in_array($name,array('.','..','.DS_Store'))) continue;
         if (is_dir($dir_path."/".$name))
           oracle_create_sp_object($dir_path."/".$name);
    }
    
}

function oracle_create_sp_object($dir_path)
{
    $dir = opendir($dir_path);
    
    $spec= "";
    $body= "";
    
    while ($name = readdir($dir))
    {
         if (substr($name,-4)=='.pls')
           $spec= $name;
         else
         if (substr($name,-4)=='.plb')
           $body= $name;
    }
    
    oracle_create_object($dir_path."/".$spec);
    
    if ($body)
         oracle_create_object($dir_path."/".$body);
}


function oracle_create_objects($dir_path)
{
    $dir = opendir($dir_path);
    
    while($name = readdir($dir))
    {
         if (in_array($name,array('.','..','.DS_Store'))) continue;
         oracle_create_object($dir_path."/".$name);
    }
    
}

function oracle_create_object($file_path)
{
         syslog(LOG_ERR,"creating object: $file_path");
         
         try
         {
            oracle_query(oracle_get_php_contents($file_path));
         }
         catch (Exception $ex)
         {
             syslog(LOG_ERR,"object $file_path created with errors");         
         }
}

function oracle_create_failsafe_objects($dir_path)
{
    $dir = opendir($dir_path);
    
    while($name = readdir($dir))
    {
         if (in_array($name,array('.','..','.DS_Store'))) continue;
         
         syslog(LOG_ERR,"creating object: $dir_path/$name");
         oracle_failsafe_ddl(oracle_get_php_contents($dir_path."/".$name));
    }
    
}

function oracle_failsafe_ddl($ddl) 
{
     oracle_query("begin execute immediate '".str_replace("'","''",$ddl)."'; exception when others then null; end;");
}

function oracle_get_php_contents($filename)
{
    if (is_file($filename))
    {
       ob_start();
       require_once $filename;
       $contents = ob_get_contents();
       ob_end_clean();
       return $contents;
    }
    else
       syslog(LOG_ERR,"error: file ".$filename." does not exists");

    return false;
}


