create or replace package body drupal_api
as
    type vc_list is table of varchar2(32767);
    type vc_arr is table of varchar2(32767) index by binary_integer;

    BLOB_PREFIX constant varchar2(3):= 'B^#';
    v_systimestamp    number:= drupal_timestamp.totimestamp(sysdate);

    single_row     constant number:= 0;
    multiple_rows  constant number:= 1;

    cursor field_type(p_name varchar2)
    is
    (  select multiple,
              db_storage
         from content_node_field
        where field_name= 'field_'||p_name);

    procedure prClobToBlob(inlob  in clob,
                      outlob in out nocopy blob) is
      /****
      *	outLob will be trimmed and filled with
      *	information from inLob
      *	default charaterset for the database will be used.
      *
      *	inlob and outLob need to be valid lob's.
      ****/

      nPos      integer;
      vcBuff    varchar2(32767);
      rBuff	    raw(32767);
      nBuffSize binary_integer;

    begin

      if	inlob is null or outlob is null then
        raise_application_error(-20000,'Invalid lob locator (null value)', true);
      end if;

      begin
        dbms_lob.trim(outLob, 0);
        npos := 0;
        loop
          nBuffSize := 32000;
          dbms_lob.read(inlob,nbuffSize,nPos +1, vcBuff);
          nPos := nPos + nBuffSize;
          rBuff := utl_raw.cast_to_raw(vcBuff);
          nBuffSize := utl_raw.length(rBuff);
          dbms_lob.writeappend(outLob, nBuffSize, rBuff);
        end loop;

           exception
              when no_data_found then -- end of lob
                 null;
      end;
    end prClobToBlob;

    procedure prblobtoclob(inlob  in blob,
                           outlob in out nocopy clob) is
      /****
      *	outLob will be trimmed and filled with
      *	information from inLob
      *	default charaterset for the database will be used.
      *
      *	inlob and outLob need to be valid lob's.
      ****/

      nPos      integer;
      vcBuff    varchar2(32767);
      rBuff	    raw(32767);
      nBuffSize binary_integer;

    begin

      if	inlob is null or outlob is null then
        raise_application_error(-20000,'Invalid lob locator (null value)', true);
      end if;

      begin
        dbms_lob.trim(outLob, 0);
        npos := 0;
        loop
          nbuffsize := 32000;
          dbms_lob.read(inlob,nbuffSize,nPos +1, rBuff);
          npos := npos + nbuffsize;
          vcbuff := utl_raw.cast_to_varchar2(rbuff);
          nBuffSize := utl_raw.length(vcBuff);
          dbms_lob.writeappend(outLob, nBuffSize, vcBuff);
        end loop;

           exception
              when no_data_found then -- end of lob
                 null;
      end;
    end prBlobToClob;



    function get_varchar2_value(p_clob in clob)
    return varchar2
    is

        v_c_length number:= dbms_lob.getlength(p_clob);
        v_length   number:= lengthb(dbms_lob.substr(p_clob, v_c_length, 1));
        blo        blobs%rowtype;

    begin

       if v_length > 4000 then

          blo := null;
          dbms_lob.createtemporary(blo.content, true, dbms_lob.call);
          dbms_lob.open(blo.content, dbms_lob.lob_readwrite);

          -- Converto il CLOB in BLOB
          prClobToBlob(p_clob, blo.content);

          -- Calcolo l'hash
          blo.hash := lower(dbms_crypto.hash(blo.content, dbms_crypto.hash_md5));

          -- Scrivo il BLOB
          begin
             select blobid into blo.blobid
               from blobs
              where hash = blo.hash;

             exception
                when no_data_found then
                   select seq_blobs.nextval into blo.blobid from dual;
                   insert into blobs values blo;
          end;

          return BLOB_PREFIX || to_char(blo.blobid);

       else

          return dbms_lob.substr(p_clob, v_c_length, 1);

       end if;

    end get_varchar2_value;

    function cleanup_clo(p_value varchar2)
    return clob
    as
      v_blobid  number;
      v_blob    blob;
      v_clob    clob;
    begin

      if p_value like blob_prefix||'%' then

        v_blobid:= to_number(substr(p_value,4));

        select content
          into v_blob
          from blobs
         where blobid= v_blobid;

        dbms_lob.createtemporary(v_clob, true, dbms_lob.call);
        dbms_lob.open(v_clob, dbms_lob.lob_readwrite);
        prblobtoclob(v_blob,v_clob);
        return v_clob;

      else
        if p_value = identifier.empty_replacer_char then
          return null;
        else
          return p_value;
        end if;
      end if;

    end;

    function cleanup_var(p_value varchar2)
    return varchar2
    as
      v_clob  clob:= cleanup_clo(p_value);
    begin
        if dbms_lob.getlength(v_clob) > 32767 then
          raise_application_error(-20001,'field value too large use a clob');
        else
          return v_clob;
        end if;
    end;

    function path_to_files
    return varchar2
    as
      v_retval varchar2(255);
      v_splits owa.vc_arr;
    begin

      select value
        into v_retval
        from "VARIABLE"
       where name= 'file_directory_path';

      v_splits:= split(v_retval,'"');

      return v_splits(2);
    end;

    function get_field_type(p_field_name varchar2)
    return field_type%rowtype
    as
    begin

      for ft in field_type(p_field_name) loop
        return ft;
      end loop;

      raise_application_error(-20001,'No field named "'||p_field_name||'" found.');
    end;


    function new_node(p_title       varchar2,
                      p_type        varchar2,
                      p_body        clob     default identifier.empty_replacer_char,
                      p_teaser      varchar2 default identifier.empty_replacer_char,
                      p_log         varchar2 default identifier.empty_replacer_char,
                      p_language    varchar2 default default_lang,
                      p_uid         number   default admin_user_uid,
                      p_status      number   default true_value,
                      p_format      number   default filtered_html_format,
                      p_created     date     default sysdate,
                      p_changed     date     default sysdate)
    return node
    as
      p_node     node;
      v_created  number:= drupal_timestamp.totimestamp(p_created);
      v_changed  number:= drupal_timestamp.totimestamp(p_changed);
      v_body     varchar2(4000):= get_varchar2_value(p_body);
      v_teaser   varchar2(4000):= get_varchar2_value(p_teaser);
    begin

        p_node.type:= lower(p_type);

        select seq_node_nid.nextval,
               seq_node_revisions_vid.nextval
          into p_node.nid,
               p_node.vid
          from dual;

         insert into node (nid, vid, "UID", "TYPE", "LANGUAGE", title, status, created, changed)
         values (p_node.nid, p_node.vid, p_uid, p_node.type, p_language, p_title, p_status, v_created, v_changed);

         insert into node_revisions
                (nid, vid, "UID", title, "BODY", teaser, "LOG", "TIMESTAMP", format)
         values (p_node.nid, p_node.vid, p_uid, p_title, v_body, v_teaser, p_log, v_created, p_format);

         begin
           execute immediate 'insert into '||identifier.get_for('content_type_'||p_node.type)||'(nid,vid) values (:nid,:vid)'
           using p_node.nid, p_node.vid;
         exception
          when others then
           if not sqlcode = -942 then -- ignore if table does not exists
              raise;
           end if;
         end;

         return p_node;
    end;

    function load_node(p_nid number)
    return node
    as
      p_node     node;
    begin

        select nid,
               vid,
               type
          into p_node.nid,
               p_node.vid,
               p_node.type
          from node
         where nid= p_nid;

         return p_node;
    end;

    function get_node_field_dml(p_node        node,
                                p_name        varchar2,
                                p_col_suffixs vc_list default vc_list('value'),
                                p_op          varchar2 default 'set')
    return vc_arr
    as
     v_field_type  field_type%rowtype:= get_field_type(p_name);
     v_table       varchar2(255);
     v_columns     vc_arr;
     v_dml         vc_arr;

     function get_col_names
     return varchar2
     as
       v_retval varchar2(2000);
     begin

        for i in 1..v_columns.count loop
          v_retval:= v_retval||','||v_columns(i);
        end loop;

        return substr(v_retval,2);

     end;

     function get_bv_names
     return varchar2
     as
       v_retval varchar2(2000);
     begin

        for i in 1..p_col_suffixs.count loop
          v_retval:= v_retval||',:'||p_col_suffixs(i);
        end loop;

        return substr(v_retval,2);

     end;

     procedure set_dml
     as
     begin

       if v_field_type.multiple = true_value then

         v_dml(1):= 'delete '||v_table||' where nid= :nid and vid= :vid';
         v_dml(2):= 'insert into '||v_table||' (nid,vid,delta,'||get_col_names||')
                          values (:nid,:vid,0,'||get_bv_names||')';

       elsif v_field_type.db_storage = false_value then

         v_dml(1):= 'delete '||v_table||' where nid= :nid and vid= :vid';
         v_dml(2):= 'insert into '||v_table||' (nid,vid,'||get_col_names||')
                          values (:nid,:vid,'||get_bv_names||')';

       else

         v_dml(1):= 'update '||v_table||' set '||v_columns(1)||'= :'||p_col_suffixs(1);

         for i in 2..v_columns.count loop
          v_dml(1):= v_dml(1)||', '||v_columns(i)||'= :'||p_col_suffixs(i);
         end loop;

         v_dml(1):= v_dml(1)||' where nid= :nid and vid= :vid';

       end if;

     end;

     procedure add_dml
     as
     begin

       if v_field_type.multiple = true_value then

         v_dml(1):= 'select nvl(max(delta),0)+1 from '||v_table||' where nid= :nid and vid= :vid';
         v_dml(2):= 'insert into '||v_table||' (nid,vid,delta,'||get_col_names||')
                          values (:nid,:vid,:delta,'||get_bv_names||')';

       else

          raise_application_error(-20001,'Cannot add values to a single-valued field ('||p_name||') use set_node_reference_field.');

       end if;

     end;

     procedure del_dml
     as
     begin

       v_dml(1):= 'delete '||v_table||' where nid= :nid and vid= :vid';

       v_dml(2):= 'delete '||v_table||' where nid= :nid and vid= :vid and '||v_columns(1)||'= :'||p_col_suffixs(1);

       for i in 2..v_columns.count loop
        v_dml(2):= v_dml(2)||', '||v_columns(i)||'= :'||p_col_suffixs(i);
       end loop;

     end;

     procedure rm_dml
     as
     begin

         v_dml(1):= 'delete '||v_table||' where nid= :nid';

     end;

    begin

       if v_field_type.multiple = true_value or v_field_type.db_storage = false_value then

         v_table:= identifier.get_for('content_field_'||p_name);

       else

         v_table:= identifier.get_for('content_type_'||p_node.type);

       end if;

       for i in 1..p_col_suffixs.count loop
         v_columns(i):= identifier.get_for('field_'||p_name||'_'||p_col_suffixs(i));
       end loop;

       if p_op = 'set' then
         set_dml;
       elsif p_op = 'add' then
         add_dml;
       elsif p_op = 'del' then
         del_dml;
       elsif p_op = 'rm' then
         rm_dml;
       else
         raise_application_error(-20001,'unknown op: '||p_op);
       end if;

       return v_dml;

    end;

    procedure remove_field(p_node node, p_name varchar2)
    as
      v_dml         vc_arr:= get_node_field_dml(p_node, p_name, vc_list(''), 'rm');
    begin

      execute immediate v_dml(1)
      using p_node.nid;

    end;

    function exists_node(p_nid number, p_type varchar2)
    return number
    as
      v_dummy number;
    begin

       select 1
         into v_dummy
         from node
        where type= p_type
          and nid= p_nid;

       return 1;
    exception
     when no_data_found then
       return 0;
    end;

    function get_nodes_by_type(p_type varchar2)
    return node_tab
    pipelined
    as
    begin

       for c_cur in (select nid from node where type= p_type) loop

          pipe row(load_node(c_cur.nid));

       end loop;

    end;

    procedure delete_node(p_node node)
    as
    begin

         for c_cur in (select field_name from content_node_field_instance where type_name= p_node.type) loop
            remove_field(p_node,substr(c_cur.field_name,7)); -- remove 'field_' from field name
         end loop;

         begin
           execute immediate 'delete '||identifier.get_for('content_type_'||p_node.type)||' where nid= :nid'
           using p_node.nid;
         exception
          when others then
           if not sqlcode = -942 then -- ignore if table does not exists
              raise;
           end if;
         end;

         delete term_node where nid= p_node.nid;
         delete node_revisions where nid= p_node.nid;
         delete node where nid= p_node.nid;

    end;

    procedure touch_node(p_node node)
    as
    begin

       update node
          set changed= v_systimestamp
        where nid= p_node.nid
          and vid= p_node.vid;

    end;

    function new_term(p_vid         number,
                      p_name        varchar2,
                      p_description varchar2,
                      p_weight      number default 0)
    return number
    as
      v_tid number;
    begin

      insert into term_data (vid, name, description, weight)
           values (p_vid, p_name, p_description, p_weight)
        returning tid into v_tid;

      insert into term_hierarchy values (v_tid, 0);

      return v_tid;

    end;

    procedure add_term_to_node(p_node node, p_tid number)
    as
    begin

       insert into term_node(nid, vid, tid)
            values (p_node.nid, p_node.vid, p_tid);

    exception
      when dup_val_on_index then -- vid + tid gia' presente: ignoro
        null;
    end;

   procedure remove_term_from_node(p_node node,
                                   p_tid  number default null,
                                   p_vid  number default null) as
   begin

      if p_tid is null and p_vid is null then
         delete term_node where nid= p_node.nid;
      else
         if p_tid is null then
            delete term_node where nid= p_node.nid and tid in (select tid from term_data where vid= p_vid);
         else
            delete term_node where nid= p_node.nid and tid= p_tid;
         end if;
      end if;
   end;

    procedure publish_node(p_node node)
    as
    begin

      update node
         set status= true_value,
             changed= v_systimestamp
       where nid = p_node.nid;

    end;

    procedure unpublish_node(p_node node)
    as
    begin

      update node
         set status= false_value,
             changed= v_systimestamp
       where nid = p_node.nid;

    end;

    procedure set_node_title(p_node node, p_title varchar2)
    as
    begin

       update node
          set title= p_title,
              changed= v_systimestamp
        where nid= p_node.nid
          and vid= p_node.vid;

       update node_revisions
          set title= p_title
        where nid= p_node.nid
          and vid= p_node.vid;

    end;

    procedure set_node_body(p_node node, p_body clob)
    as
      v_body varchar2(4000):= get_varchar2_value(p_body);
    begin

       update node_revisions
          set body= v_body
        where nid= p_node.nid
          and vid= p_node.vid;

       touch_node(p_node);

    end;

    function get_node_title(p_node node)
    return varchar2
    as
      v_retval varchar2(4000);
    begin

       select title
         into v_retval
         from node
        where nid= p_node.nid
          and vid= p_node.vid;

       return cleanup_var(v_retval);

    end;

    function get_node_body(p_node node)
    return clob
    as
      v_retval clob;
    begin

       select body
         into v_retval
         from node_revisions
        where nid= p_node.nid
          and vid= p_node.vid;

       return cleanup_clo(v_retval);

    end;

    procedure set_node_field(p_node node, p_name varchar2, p_value clob)
    as
     v_dml         vc_arr:= get_node_field_dml(p_node, p_name);
    begin

       if v_dml.count = 2 then

         execute immediate v_dml(1)
         using p_node.nid, p_node.vid;

         execute immediate v_dml(2)
         using p_node.nid, p_node.vid, get_varchar2_value(p_value);

       else

         execute immediate v_dml(1)
         using get_varchar2_value(p_value), p_node.nid, p_node.vid;

       end if;

       touch_node(p_node);

    end;

    procedure set_node_field(p_node node, p_name varchar2, p_value varchar2)
    as
    begin
       set_node_field(p_node, p_name, to_clob(p_value));
    end;

    procedure set_node_field(p_node node, p_name varchar2, p_value number)
    as
     v_dml         vc_arr:= get_node_field_dml(p_node, p_name);
    begin

       if v_dml.count = 2 then

         execute immediate v_dml(1)
         using p_node.nid, p_node.vid;

         execute immediate v_dml(2)
         using p_node.nid, p_node.vid, p_value;

       else

         execute immediate v_dml(1)
         using p_value, p_node.nid, p_node.vid;

       end if;

       touch_node(p_node);

    end;

    procedure set_node_field(p_node node, p_name varchar2, p_value date)
    as
    begin
      set_node_field(p_node,p_name,drupal_timestamp.totimestamp(p_value));
    end;


    $if dbms_db_version.version > 10 $then
    procedure set_node_fields(p_node node, p_cur in out sys_refcursor)
    as
        v_row  dynamic_cursor.associative_row;
        v_col  varchar2(255);
    begin

        dynamic_cursor.open(p_cur);

        if dynamic_cursor.fetch_row(v_row) then

         v_col:= v_row.first;

         while v_col is not null loop

            set_node_field(p_node,lower(v_col),v_row(v_col));

            v_col:= v_row.next(v_col);

         end loop;

        else
          raise no_data_found;
        end if;

        dynamic_cursor.close;

    end;
    $end


    procedure set_node_link_field(p_node       node,
                                  p_name       varchar2,
                                  p_url        varchar2,
                                  p_title      varchar2,
                                  p_attributes varchar2 default null)
    as
     v_dml         vc_arr:= get_node_field_dml(p_node, p_name, vc_list('url','title','attributes'));
    begin

       if v_dml.count = 2 then

         execute immediate v_dml(1)
         using p_node.nid, p_node.vid;

         execute immediate v_dml(2)
         using p_node.nid, p_node.vid, p_url, p_title, p_attributes;

       else

         execute immediate v_dml(1)
         using p_url, p_title, p_attributes, p_node.nid, p_node.vid;

       end if;

       touch_node(p_node);

    end;

    procedure set_node_reference_field(p_node node, p_name varchar2, p_nid number)
    as
     v_dml         vc_arr:= get_node_field_dml(p_node, p_name, vc_list('nid'));
    begin

       if v_dml.count = 2 then

         execute immediate v_dml(1)
         using p_node.nid, p_node.vid;

         execute immediate v_dml(2)
         using p_node.nid, p_node.vid, p_nid;

       else

         execute immediate v_dml(1)
         using p_nid, p_node.nid, p_node.vid;

       end if;

       touch_node(p_node);

    end;

    procedure add_node_reference_field(p_node node, p_name varchar2, p_nid number)
    as
     v_dml         vc_arr:= get_node_field_dml(p_node, p_name, vc_list('nid'), 'add');
     v_delta       number;
    begin

         execute immediate v_dml(1)
         into v_delta
         using p_node.nid, p_node.vid;

         execute immediate v_dml(2)
         using p_node.nid, p_node.vid, v_delta, p_nid;

         touch_node(p_node);

    end;

    procedure del_node_reference_field(p_node node, p_name varchar2, p_nid number default null)
    as
     v_dml         vc_arr:= get_node_field_dml(p_node, p_name, vc_list('nid'), 'del');
     v_delta       number;
    begin

       if p_nid is null then
         execute immediate v_dml(1)
         using p_node.nid, p_node.vid;
       else
         execute immediate v_dml(2)
         using p_node.nid, p_node.vid, p_nid;
       end if;

       touch_node(p_node);

    end;

    procedure save_file(p_file in out file)
    as
    begin

      if p_file.fid is null then

       insert into files (fid,
                          "UID",
                          filename,
                          filepath,
                          filemime,
                          filesize,
                          status,
                          timestamp)
            values (seq_files_fid.nextval,
                    p_file.user_id,
                    p_file.name,
                    p_file.path,
                    p_file.mimetype,
                    p_file.bsize,
                    p_file.status,
                    drupal_timestamp.totimestamp(p_file.time_stamp))
          returning fid into p_file.fid;

      else

          update files
             set "UID"= p_file.user_id,
                  filename= p_file.name,
                  filepath= p_file.path,
                  filemime= p_file.mimetype,
                  filesize= p_file.bsize,
                  status= p_file.status,
                  timestamp= drupal_timestamp.totimestamp(p_file.time_stamp)
            where fid= p_file.fid;

      end if;

    end;

    procedure set_node_file_field(p_node node, p_name varchar2, p_value file_field)
    as
      v_file_field  file_field:= p_value;
      v_name        varchar2(255):= lower(p_name);
      v_dml         vc_arr:= get_node_field_dml(p_node, p_name, vc_list('fid','list','data'));

      procedure normalize_file_field
      as
      begin

         if v_file_field.list is null then
          v_file_field.list:= 1;
         end if;

         if v_file_field.data is null then
          v_file_field.data:= 'a:3:{s:11:"description";s:0:"";s:3:"alt";s:0:"";s:5:"title";s:0:"";}';
         end if;

      end;

    begin

       normalize_file_field;

       if v_dml.count = 2 then

         execute immediate v_dml(1)
         using p_node.nid, p_node.vid;

         execute immediate v_dml(2)
         using p_node.nid, p_node.vid, v_file_field.ifile.fid, v_file_field.list, v_file_field.data;

       else

         execute immediate v_dml(1)
         using v_file_field.ifile.fid, v_file_field.list, v_file_field.data, p_node.nid, p_node.vid;

       end if;

       touch_node(p_node);

    end;

    function get_node_field_query(p_node        node,
                                  p_name        varchar2,
                                  p_col_suffixs vc_list default vc_list('value'),
                                  p_cardinality number  default single_row)
    return varchar2
    as
      v_field_type  field_type%rowtype:= get_field_type(p_name);
      v_name        varchar2(255):= lower(p_name);
      v_table       varchar2(255);
      v_columns     vc_arr;
      v_dml         vc_arr;
      v_query       varchar2(2000);

      function get_col_names
      return varchar2
      as
       v_retval varchar2(2000);
      begin

        for i in 1..v_columns.count loop
          v_retval:= v_retval||','||v_columns(i);
        end loop;

        return substr(v_retval,2);

      end;


    begin

       for i in 1..p_col_suffixs.count loop
         v_columns(i):= identifier.get_for('field_'||v_name||'_'||p_col_suffixs(i));
       end loop;

       if v_field_type.multiple= true_value or v_field_type.db_storage= false_value  then

         v_table:= identifier.get_for('content_field_'||v_name);

       else

         v_table:= identifier.get_for('content_type_'||p_node.type);

       end if;

       v_query:= 'select '||get_col_names||' from '||v_table||' where nid= :nid and vid= :vid';

       if v_field_type.multiple = true_value then
         v_query:= v_query||' order by delta';
       end if;

       if p_cardinality = single_row then
         v_query:= 'select * from ('||v_query||') where rownum < 2';
       end if;

       return v_query;

    end;

    function get_node_by_value_query(p_node_type   varchar2,
                                     p_field_name  varchar2,
                                     p_col_suffixs vc_list default vc_list('value'),
                                     p_cardinality number  default single_row)
    return varchar2
    as
      v_field_type  field_type%rowtype:= get_field_type(p_field_name);
      v_name        varchar2(255):= lower(p_field_name);
      v_table       varchar2(255);
      v_columns     vc_arr;
      v_dml         vc_arr;
      v_query       varchar2(2000);

      function get_col_filters
      return varchar2
      as
       v_retval varchar2(2000);
      begin

        for i in 1..v_columns.count loop
          v_retval:= v_retval||' and '||v_columns(i)||'= :'||p_col_suffixs(i);
        end loop;

        return substr(v_retval,6);

      end;


    begin

       for i in 1..p_col_suffixs.count loop
         v_columns(i):= identifier.get_for('field_'||v_name||'_'||p_col_suffixs(i));
       end loop;

       if v_field_type.multiple= true_value or v_field_type.db_storage= false_value  then

         v_table:= identifier.get_for('content_field_'||v_name);

       else

         v_table:= identifier.get_for('content_type_'||p_node_type);

       end if;

       v_query:= 'select nid from '||v_table||' where '||get_col_filters;

       if v_field_type.multiple = true_value then
         v_query:= v_query||' order by delta';
       end if;

       if p_cardinality = single_row then
         v_query:= 'select * from ('||v_query||') where rownum < 2';
       end if;

       return v_query;

    end;

    function load_file(p_fid number)
    return file
    as
      v_file file;
    begin

      begin
        select fid,
               filepath,
               filename,
               filesize,
               filemime,
               drupal_timestamp.todate("TIMESTAMP"),
               status,
               "UID"
          into v_file.fid,
               v_file.path,
               v_file.name,
               v_file.bsize,
               v_file.mimetype,
               v_file.time_stamp,
               v_file.status,
               v_file.user_id
          from files
         where fid= p_fid;
      exception
       when no_data_found then
         null;
      end;

      return v_file;

    end;


    function get_node_file_field(p_node       node,
                                 p_name       varchar2)
    return file_field
    as
      v_value       file_field;
    begin

       begin
         execute immediate get_node_field_query(p_node,p_name,vc_list('fid','list','data'))
         into v_value.ifile.fid, v_value.list, v_value.data
         using p_node.nid, p_node.vid;
       exception
        when no_data_found then
         null;
       end;

       if v_value.ifile.fid is not null then
         v_value.ifile:= load_file(v_value.ifile.fid);
       end if;

       return v_value;

    end;

    function get_node_clo_field(p_node node, p_name varchar2)
    return clob
    as
      v_value       clob;
    begin

       execute immediate get_node_field_query(p_node,p_name)
       into v_value
       using p_node.nid, p_node.vid;

       return cleanup_clo(v_value);

    end;

    function get_node_num_field(p_node node, p_name varchar2)
    return number
    as
      v_value       number;
    begin

       execute immediate get_node_field_query(p_node,p_name)
       into v_value
       using p_node.nid, p_node.vid;

      return v_value;
    end;

    function get_node_var_field(p_node node, p_name varchar2)
    return varchar2
    as
      v_value       varchar2(32767);
    begin

       execute immediate get_node_field_query(p_node,p_name)
       into v_value
       using p_node.nid, p_node.vid;

      return cleanup_var(v_value);
    end;

    function get_node_email_field(p_node node, p_name varchar2)
    return varchar2
    as
      v_value       varchar2(32767);
    begin

       execute immediate get_node_field_query(p_node,p_name,vc_list('email'))
       into v_value
       using p_node.nid, p_node.vid;

      return cleanup_var(v_value);
    end;


    function get_node_dat_field(p_node node, p_name varchar2)
    return date
    as
    begin
      return drupal_timestamp.todate(get_node_num_field(p_node, p_name));
    end;

    function get_node_term_field(p_node node, p_name varchar2)
    return varchar2
    as
       v_tid   number:= drupal_api.get_node_num_field(p_node,p_name);
       v_term  varchar2(4000);
    begin

       select name
         into v_term
         from term_data
        where tid= v_tid;

       return v_term;

    end;


    function get_referenced_nodes(p_reference_field_name varchar2)
    return node_tab
    pipelined
    as
      v_nid    number;
      c_cur    sys_refcursor;
    begin

      open c_cur for get_node_field_query(loop_node,p_reference_field_name, vc_list('nid'), multiple_rows)
      using loop_node.nid, loop_node.vid;
      loop
        fetch c_cur into v_nid;
        exit when c_cur%notfound;

        pipe row (load_node(v_nid));

      end loop;
      close c_cur;

    end;

    function get_referenced_node(p_node node, p_reference_field_name varchar2)
    return node
    as
      v_nid    number;
    begin

      execute immediate get_node_field_query(p_node, p_reference_field_name, vc_list('nid'))
      into v_nid
      using p_node.nid, p_node.vid;

      return load_node(v_nid);

    end;

    function get_nodes_by_reference_value(p_node_type varchar2, p_reference_field_name varchar2, p_nid number)
    return node_tab
    pipelined
    as
      v_nid    number;
      c_cur    sys_refcursor;
    begin
      open c_cur for get_node_by_value_query(p_node_type, p_reference_field_name, vc_list('nid'), multiple_rows)
      using p_nid;
      loop
        fetch c_cur into v_nid;
        exit when c_cur%notfound;

        pipe row (load_node(v_nid));

      end loop;
      close c_cur;
    end;

    function get_node_by_reference_value(p_node_type varchar2, p_reference_field_name varchar2, p_nid number)
    return node
    as
      v_nid    number;
    begin

      execute immediate get_node_by_value_query(p_node_type, p_reference_field_name, vc_list('nid'))
      into v_nid
      using p_nid;

      return load_node(v_nid);

    exception
      when no_data_found then
        return empty_node;
    end;

    procedure set_node_email_field(p_node node, p_name varchar2, p_email varchar2)
    as
      v_dml         vc_arr:= get_node_field_dml(p_node, p_name, vc_list('email'));
    begin

       if v_dml.count = 2 then

         execute immediate v_dml(1)
         using p_node.nid, p_node.vid;

         execute immediate v_dml(2)
         using p_node.nid, p_node.vid, p_email;

       else

         execute immediate v_dml(1)
         using p_email, p_node.nid, p_node.vid;

       end if;

       touch_node(p_node);

    end;

    procedure add_node_email_field(p_node node, p_name varchar2, p_email varchar2)
    as
      v_dml         vc_arr:= get_node_field_dml(p_node, p_name, vc_list('email'),'add');
      v_delta       number;
    begin

         execute immediate v_dml(1)
         into v_delta
         using p_node.nid, p_node.vid;

         execute immediate v_dml(2)
         using p_node.nid, p_node.vid, v_delta, p_email;

         touch_node(p_node);

    end;


end drupal_api;

