create or replace package drupal_api
as

    admin_user_uid   constant  number:= 1;
    default_lang     constant  varchar2(2):= 'it';

    true_value    constant  number:= 1;
    false_value   constant  number:= 0;

    php_format              constant  number:= 21;
    filtered_html_format    constant  number:= 1;
    full_html_format        constant  number:= 2;

    type node is record
    (
         nid     number,
         vid     number,
         type    varchar2(255)
    );

    type file is record
    (
         fid          number,
         path         varchar2(2000),
         name         varchar2(255),
         bsize        number,
         mimetype     varchar2(255),
         time_stamp   date,
         status       number,
         user_id      number
    );

    type file_field is record
    (
         ifile     file,
         list      number,
         data      varchar2(4000)
    );

    type file_field_arr is table of file_field index by binary_integer;
    type node_tab is table of node;

    empty_node node;

    loop_node  node;

    function new_term(p_vid         number,
                      p_name        varchar2,
                      p_description varchar2,
                      p_weight      number default 0)
    return number;

    procedure add_term_to_node(p_node node, p_tid number);

    procedure remove_term_from_node(p_node node,
                                    p_tid  number default null,
                                    p_vid  number default null);

    function get_node_term_field(p_node node, p_name varchar2)
    return varchar2;

    function new_node(p_title       varchar2,
                      p_type        varchar2,
                      p_body        clob     default identifier.empty_replacer_char,
                      p_teaser      varchar2 default identifier.empty_replacer_char,
                      p_log         varchar2 default identifier.empty_replacer_char,
                      p_language    varchar2 default default_lang,
                      p_uid         number   default admin_user_uid,
                      p_status      number   default true_value,
                      p_format      number   default filtered_html_format,
                      p_created     date     default sysdate,
                      p_changed     date     default sysdate)
    return node;

    function load_node(p_nid number)
    return node;

    procedure publish_node(p_node node);

    procedure unpublish_node(p_node node);

    procedure delete_node(p_node node);

    procedure set_node_title(p_node node, p_title varchar2);

    function get_node_title(p_node node)
    return varchar2;

    procedure set_node_body(p_node node, p_body clob);

    function get_node_body(p_node node)
    return clob;

    function exists_node(p_nid number, p_type varchar2)
    return number;

    function get_nodes_by_type(p_type varchar2)
    return node_tab
    pipelined;

    procedure set_node_field(p_node node, p_name varchar2, p_value clob);

    procedure set_node_field(p_node node, p_name varchar2, p_value number);

    procedure set_node_field(p_node node, p_name varchar2, p_value varchar2);

    procedure set_node_field(p_node node, p_name varchar2, p_value date);

    function get_node_clo_field(p_node node, p_name varchar2)
    return clob;

    function get_node_num_field(p_node node, p_name varchar2)
    return number;

    function get_node_var_field(p_node node, p_name varchar2)
    return varchar2;

    function get_node_dat_field(p_node node, p_name varchar2)
    return date;

    function get_node_email_field(p_node node, p_name varchar2)
    return varchar2;

    procedure set_node_email_field(p_node node, p_name varchar2, p_email varchar2);

    procedure add_node_email_field(p_node node, p_name varchar2, p_email varchar2);

    procedure set_node_reference_field(p_node node, p_name varchar2, p_nid number);

    procedure add_node_reference_field(p_node node, p_name varchar2, p_nid number);

    procedure del_node_reference_field(p_node node, p_name varchar2, p_nid number default null);

    function get_referenced_nodes(p_reference_field_name varchar2)
    return node_tab
    pipelined;

    function get_referenced_node(p_node node, p_reference_field_name varchar2)
    return node;

    function get_nodes_by_reference_value(p_node_type varchar2, p_reference_field_name varchar2, p_nid number)
    return node_tab
    pipelined;

    function get_node_by_reference_value(p_node_type varchar2, p_reference_field_name varchar2, p_nid number)
    return node;

    $if dbms_db_version.version > 10 $then
    procedure set_node_fields(p_node node, p_cur in out sys_refcursor);
    $end

    procedure set_node_link_field(p_node       node,
                                  p_name       varchar2,
                                  p_url        varchar2,
                                  p_title      varchar2,
                                  p_attributes varchar2 default null);


    procedure set_node_file_field(p_node       node,
                                  p_name       varchar2,
                                  p_value      file_field);

    function get_node_file_field(p_node       node,
                                 p_name       varchar2)
    return file_field;

    function load_file(p_fid number)
    return file;

    procedure save_file(p_file in out file);

    function path_to_files
    return varchar2;

end drupal_api;

