create or replace package body dynamic_cursor
as
   
   max_cursor_column_count  constant pls_integer:= 255;
   
   v_cursor         pls_integer; 
   v_col_count      pls_integer; 
   v_col_desc       dbms_sql.desc_tab;
   
   type vc_ass   is table of varchar2(4000) index by varchar2(255);
   type num_ass  is table of varchar2(4000) index by varchar2(255);
   type date_ass is table of varchar2(4000) index by varchar2(255);
   
   vc_vars     vc_ass;
   num_vars    num_ass;
   date_vars   date_ass;
          
   procedure open_helper
   as
     v_col_value      varchar2(30);
   begin

     for i in 1..max_cursor_column_count loop
     
         begin
             dbms_sql.define_column(v_cursor, i, v_col_value, 4000);
             v_col_count := i;
         exception
             when others then
                 if sqlcode = -1007 then
                     exit;
                 else
                     raise;
                 end if;
         end;
         
     end loop;

     dbms_sql.describe_columns(v_cursor, v_col_count, v_col_desc);
     
     for i in 1..v_col_count loop
         v_col_desc(i).col_name:= lower(v_col_desc(i).col_name);
     end loop;

   end;
          
   procedure open(p_sql varchar2)
   as
     v_status         number;
     v_processed_rows number;
   begin
   
     v_cursor:= dbms_sql.open_cursor;
     dbms_sql.parse(v_cursor,  p_sql, dbms_sql.native );
      
     v_processed_rows := dbms_sql.execute(v_cursor);
     
     open_helper;
      
   end;
   
   $if dbms_db_version.version > 10 $then
   procedure open(p_ref_cursor in out sys_refcursor)
   as
   begin
   
     v_cursor:= dbms_sql.to_cursor_number(p_ref_cursor);
      
     open_helper;
     
   end;
   $end
   
   function fetch_row(p_row in out associative_row)
   return boolean
   as
     v_row          associative_row;
     v_column_value varchar2(4000);
     v_fetched_rows number;
   begin
   
     v_row:= empty_row;
   
     v_fetched_rows:= dbms_sql.fetch_rows(v_cursor);
     
     for i in 1..v_col_count loop
         dbms_sql.column_value(v_cursor, i, v_column_value);
         p_row(v_col_desc(i).col_name):= v_column_value;
     end loop;


     return (v_fetched_rows!=0);
   end;

   procedure close
   as
   begin
      dbms_sql.close_cursor(v_cursor);
   end;
   
   procedure bind_variable(p_name varchar2, p_value varchar2)
   as
   begin
      vc_vars(p_name):= p_value;
   end;
   
   procedure bind_variable(p_name varchar2, p_value number)
   as
   begin
      num_vars(p_name):= p_value;
   end;

   procedure bind_variable(p_name varchar2, p_value date)
   as
   begin
      date_vars(p_name):= p_value;
   end;
   
   function string_variable(p_name varchar2)
   return varchar2
   deterministic
   as
   begin
      return vc_vars(p_name);
   exception
    when no_data_found then
      return null;
   end;

   function number_variable(p_name varchar2)
   return number
   deterministic
   as
   begin
      return num_vars(p_name);
   exception
    when no_data_found then
      return null;
   end;

   function date_variable(p_name varchar2)
   return date
   deterministic
   as
   begin
      return date_vars(p_name);
   exception
    when no_data_found then
      return null;
   end;
   
   
   function get_column_desc
   return dbms_sql.desc_tab
   as
   begin
     return v_col_desc;
   end;
   
end;

